@@ -29,12 +29,13 @@ struct kerndat_s {
 	bool has_dirty_track;
 	bool has_memfd;
 	bool has_fdinfo_lock;
-	bool has_tcp_window;
 	unsigned long task_size;
 	bool ipv6;
 	bool has_loginuid;
 	enum pagemap_func pmap;
 	unsigned int has_xtlocks;
+	unsigned long mmap_min_addr;
+	bool has_tcp_half_closed;
 };
 
 extern struct kerndat_s kdat;
@@ -55,6 +56,6 @@ enum {
  */
 extern int kerndat_fs_virtualized(unsigned int which, u32 kdev);
 
-extern int kerndat_tcp_repair_window();
+extern int kerndat_tcp_repair();
 
 #endif /* __CR_KERNDAT_H__ */
