@@ -3,32 +3,13 @@
 
 #include <sys/types.h>
 
-#include "asm/types.h"
-#include "list.h"
+#include "common/list.h"
 
 struct proc_mountinfo;
 struct pstree_item;
 struct fstype;
 struct ns_id;
 
-/*
- * Structure to keep external mount points resolving info.
- *
- * On dump the key is the mountpoint as seen from the mount
- * namespace, the val is some name that will be put into image
- * instead of the mount point's root path.
- *
- * On restore the key is the name from the image (the one
- * mentioned above) and the val is the path in criu's mount
- * namespace that will become the mount point's root, i.e. --
- * be bind mounted to the respective mountpoint.
- */
-struct ext_mount {
-	struct list_head	list;
-	char			*key;
-	char			*val;
-};
-
 #define MOUNT_INVALID_DEV	(0)
 
 struct mount_info {
@@ -68,7 +49,7 @@ struct mount_info {
 	struct mount_info	*next;
 	struct ns_id		*nsid;
 
-	struct ext_mount	*external;
+	char			*external;
 	bool			internal_sharing;
 
 	/* tree linkage */
@@ -90,7 +71,11 @@ struct mount_info {
 
 extern struct mount_info *mntinfo;
 extern struct ns_desc mnt_ns_desc;
+#ifdef CONFIG_BINFMT_MISC_VIRTUALIZED
 extern int collect_binfmt_misc(void);
+#else
+static inline int collect_binfmt_misc(void) { return 0; }
+#endif
 
 extern struct mount_info *mnt_entry_alloc();
 extern void mnt_entry_free(struct mount_info *mi);
@@ -124,6 +109,7 @@ extern int depopulate_roots_yard(int mntns_root, bool clean_remaps);
 
 extern int rst_get_mnt_root(int mnt_id, char *path, int plen);
 extern int ext_mount_add(char *key, char *val);
+extern int ext_mount_parse_auto(char *key);
 extern int mntns_maybe_create_roots(void);
 extern int read_mnt_ns_img(void);
 extern void cleanup_mnt_ns(void);
