@@ -1,7 +1,6 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <stdlib.h>
 #include <sys/mman.h>
 
@@ -14,8 +13,10 @@
 #include "autofs.h"
 
 #include "protobuf.h"
+#include "util.h"
 #include "images/pipe.pb-c.h"
 #include "images/pipe-data.pb-c.h"
+#include "fcntl.h"
 
 static LIST_HEAD(pipes);
 
